import discord
from discord.ext import commands
from discord import app_commands
import os
import requests
from dotenv import load_dotenv
import logging
import json

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s:%(levelname)s:%(name)s: %(message)s',
    handlers=[
        logging.FileHandler("bot.log"),
        logging.StreamHandler()
    ]
)
logger = logging.getLogger('discord_bot')

load_dotenv()
TOKEN = os.getenv('DISCORD_TOKEN')
CHANNEL_ID = int(os.getenv('CHANNEL_ID'))
LOG_CHANNEL_ID = int(os.getenv('LOG_CHANNEL_ID'))  
PT_API_URL = os.getenv('PT_API_URL').rstrip('/') 
PT_API_KEY = os.getenv('PT_API_KEY')

try:
    with open('servers.json', 'r') as f:
        servers_config = json.load(f)
    SERVERS = {server['uuid']: server['name'] for server in servers_config['servers']}
except Exception as e:
    logger.critical(f"Failed to load servers configuration: {e}")
    SERVERS = {}


if not SERVERS:
    logger.critical("No servers configured. Please add servers to 'servers.json' or define them in the code.")
    exit(1)

PT_HEADERS = {
    'Authorization': f'Bearer {PT_API_KEY}',
    'Content-Type': 'application/json',
    'Accept': 'Application/vnd.pterodactyl.v1+json'
}

intents = discord.Intents.default()
intents.message_content = True  

bot = commands.Bot(command_prefix="!", intents=intents)

async def log_action(action, server_name, user):
    """
    Sends a log message to the designated logging channel.

    Parameters:
    - action (str): The action performed (start, kill, restart, save).
    - server_name (str): The name of the server on which the action was performed.
    - user (discord.User): The Discord user who performed the action.
    """
    try:
        logger.info(f"Attempting to log action: {action} on server: {server_name} by user: {user}")
        log_channel = bot.get_channel(LOG_CHANNEL_ID)
        if not log_channel:
            logger.error(f"Logging channel with ID {LOG_CHANNEL_ID} not found.")
            return

        timestamp = discord.utils.utcnow().strftime("%Y-%m-%d %H:%M:%S UTC")
        embed = discord.Embed(
            title="Server Action Performed",
            color=discord.Color.blue(),
            timestamp=discord.utils.utcnow()
        )
        embed.add_field(name="Action", value=action.capitalize(), inline=True)
        embed.add_field(name="Server", value=server_name, inline=True)
        embed.add_field(name="User", value=f"{user} ({user.id})", inline=False)
        embed.set_footer(text=f"Timestamp: {timestamp}")

        await log_channel.send(embed=embed)
        logger.info(f"Successfully logged action: {action} on server: {server_name} by user: {user}")
    except Exception as e:
        logger.error(f"Failed to send log message: {e}")

class ServerControlView(discord.ui.View):
    def __init__(self, servers):
        super().__init__(timeout=None) 
        self.servers = servers 
        self.selected_server = None 

        self.add_item(ServerSelect(servers))

    @discord.ui.button(label="Start Server", style=discord.ButtonStyle.green, custom_id="start_server")
    async def start_button(self, interaction: discord.Interaction, button: discord.ui.Button):
        await interaction.response.defer(ephemeral=True)
        if not self.selected_server:
            await interaction.followup.send("❌ Please select a server first.", ephemeral=True)
            return

        if not interaction.user.guild_permissions.administrator:
            await interaction.followup.send("❌ You do not have permission to perform this action.", ephemeral=True)
            return

        await self.perform_power_action(interaction, "start")

    @discord.ui.button(label="Kill Server", style=discord.ButtonStyle.red, custom_id="kill_server")
    async def kill_button(self, interaction: discord.Interaction, button: discord.ui.Button):
        await interaction.response.defer(ephemeral=True)
        if not self.selected_server:
            await interaction.followup.send("❌ Please select a server first.", ephemeral=True)
            return

        if not interaction.user.guild_permissions.administrator:
            await interaction.followup.send("❌ You do not have permission to perform this action.", ephemeral=True)
            return

        await interaction.followup.send(
            "⚠️ **Are you sure you want to kill the server?** This action cannot be undone.",
            view=KillConfirmationView(self.selected_server),
            ephemeral=True
        )

    @discord.ui.button(label="Restart Server", style=discord.ButtonStyle.blurple, custom_id="restart_server")
    async def restart_button(self, interaction: discord.Interaction, button: discord.ui.Button):
        await interaction.response.defer(ephemeral=True)
        if not self.selected_server:
            await interaction.followup.send("❌ Please select a server first.", ephemeral=True)
            return

        if not interaction.user.guild_permissions.administrator:
            await interaction.followup.send("❌ You do not have permission to perform this action.", ephemeral=True)
            return

        await self.perform_power_action(interaction, "restart")

    @discord.ui.button(label="Save Server", style=discord.ButtonStyle.gray, custom_id="save_server")
    async def save_button(self, interaction: discord.Interaction, button: discord.ui.Button):
        await interaction.response.defer(ephemeral=True)
        if not self.selected_server:
            await interaction.followup.send("❌ Please select a server first.", ephemeral=True)
            return

        if not interaction.user.guild_permissions.administrator:
            await interaction.followup.send("❌ You do not have permission to perform this action.", ephemeral=True)
            return

        await self.perform_save_action(interaction)

    async def perform_power_action(self, interaction, action):
        url = f"{PT_API_URL}/client/servers/{self.selected_server}/power"
        payload = {"signal": action}

        try:
            response = requests.post(url, json=payload, headers=PT_HEADERS)
            logger.info(f"{action.capitalize()} Server Response: {response.status_code} - {response.text}")

            if response.status_code == 204:
                status_emojis = {
                    "start": "🚀",
                    "kill": "💀",
                    "restart": "🔄"
                }
                await interaction.followup.send(f"{status_emojis.get(action, '')} Rust server is {action}ing...", ephemeral=True)
                server_name = self.servers.get(self.selected_server, "Unknown Server")
                await log_action(action, server_name, interaction.user)

                if response.status_code == 204:
                    status_emojis = {
                        "start": "🚀",
                        "kill": "💀",
                        "restart": "🔄"
                    }
                    await interaction.followup.send(f"{status_emojis.get(action, '')} Rust server is {action}ing...", ephemeral=True)
                    server_name = self.servers.get(self.selected_server, "Unknown Server")
                    await log_action(action, server_name, interaction.user)
                else:
                    await interaction.followup.send(f"❌ Failed to {action} server: {response.text}", ephemeral=True)
            else:
                await interaction.followup.send(f"❌ Failed to {action} server: {response.text}", ephemeral=True)
        except Exception as e:
            logger.error(f"Error performing {action} action on server {self.selected_server}: {e}")
            await interaction.followup.send(f"❌ An error occurred: {e}", ephemeral=True)

    async def perform_save_action(self, interaction):
        url = f"{PT_API_URL}/client/servers/{self.selected_server}/command"
        payload = {"command": "save"}

        try:
            response = requests.post(url, json=payload, headers=PT_HEADERS)
            logger.info(f"Save Server Response: {response.status_code} - {response.text}")

            if response.status_code == 201:
                await interaction.followup.send("💾 Rust server save command has been executed.", ephemeral=True)
                server_name = self.servers.get(self.selected_server, "Unknown Server")
                await log_action("save", server_name, interaction.user)
            elif response.status_code == 429:
                await interaction.followup.send("⚠️ Rate limited. Please try again later.", ephemeral=True)
        except Exception as e:
            logger.error(f"Error saving server {self.selected_server}: {e}")
            await interaction.followup.send(f"❌ An error occurred: {e}", ephemeral=True)

class KillConfirmationView(discord.ui.View):
    def __init__(self, server_uuid):
        super().__init__(timeout=30)  
        self.server_uuid = server_uuid

    @discord.ui.button(label="Yes, Kill", style=discord.ButtonStyle.danger, custom_id="confirm_kill")
    async def confirm_kill(self, interaction: discord.Interaction, button: discord.ui.Button):
        url = f"{PT_API_URL}/client/servers/{self.server_uuid}/power"
        payload = {"signal": "kill"}

        try:
            response = requests.post(url, json=payload, headers=PT_HEADERS)
            logger.info(f"Kill Server Response: {response.status_code} - {response.text}")

            if response.status_code == 204:
                await interaction.followup.send("💀 Rust server has been killed.", ephemeral=True)
                server_name = SERVERS.get(self.server_uuid, "Unknown Server")
                await log_action("kill", server_name, interaction.user)

                if response.status_code == 204:
                    await interaction.followup.send("💀 Rust server has been killed.", ephemeral=True)
                    server_name = SERVERS.get(self.server_uuid, "Unknown Server")
                    await log_action("kill", server_name, interaction.user)
                else:
                    await interaction.followup.send(f"❌ Failed to kill server: {response.text}", ephemeral=True)
            else:
                await interaction.followup.send(f"❌ Failed to kill server: {response.text}", ephemeral=True)
        except Exception as e:
            logger.error(f"Error killing server {self.server_uuid}: {e}")
            await interaction.followup.send(f"❌ An error occurred: {e}", ephemeral=True)

        self.stop()  

    @discord.ui.button(label="No, Cancel", style=discord.ButtonStyle.secondary, custom_id="cancel_kill")
    async def cancel_kill(self, interaction: discord.Interaction, button: discord.ui.Button):
        await interaction.response.send_message("✅ Kill action canceled.", ephemeral=True)
        self.stop()

class ServerSelect(discord.ui.Select):
    def __init__(self, servers):
        options = [
            discord.SelectOption(label=name, value=uuid)
            for uuid, name in servers.items()
        ]
        super().__init__(
            custom_id="server_select",  
            placeholder="Select a server to manage...",
            min_values=1,
            max_values=1,
            options=options
        )
        self.servers = servers

    async def callback(self, interaction: discord.Interaction):
        selected_uuid = self.values[0]
        self.view.selected_server = selected_uuid
        selected_name = self.servers.get(selected_uuid, "Unknown Server")
        await interaction.response.send_message(f"✅ Selected server: **{selected_name}**", ephemeral=True)

@bot.event
async def on_ready():
    logger.info(f'Logged in as {bot.user} (ID: {bot.user.id})')
    logger.info('------')
    bot.add_view(ServerControlView(SERVERS))

    channel = bot.get_channel(CHANNEL_ID)
    if channel:
        try:
            async for message in channel.history(limit=100):
                if message.author == bot.user and "Control the Rust server" in message.content:
                    logger.info("Control message already exists.")
                    break
            else:
                await channel.send("Control the Rust server using the dropdown and buttons below:", view=ServerControlView(SERVERS))
                logger.info("Control message sent.")
        except Exception as e:
            logger.error(f"Error checking messages: {e}")
    else:
        logger.error(f"Channel with ID {CHANNEL_ID} not found.")

@bot.event
async def setup_hook():
    await bot.tree.sync()
    logger.info("Slash commands synced.")

try:
    bot.run(TOKEN)
except Exception as e:
    logger.critical(f"Failed to run the bot: {e}")
